local cc = {}
cc.version = '0.2'

function cc.create_colorpicker_dialog(args)
	local dlg, container
	local title_label
	local previewbox
	local red_text, green_text, blue_text
	local ok_button, cancel_button, undo_button, default_button
	
	local last_color
	local current_color = args.color and tostring(args.color) or "0 0 0"
	local default_color = args.default_color and tostring(args.default_color) or "0 0 0"
	
	local title = args.title and tostring(args.title) or "Color Picker"
	local default_name = args.default_name and tostring(args.default_name) or "Default"
	
	local function set_color(color)
		local red, green, blue = color:match("^(%d+) (%d+) (%d+)")
		
		previewbox.bgcolor = color
		current_color = color
		
		red_text.value = red
		green_text.value = green
		blue_text.value = blue
	end
	
	title_label = iup.label{title=title, font=Font.H3}
	previewbox = iup.label{title="", expand="YES"}
	
	local function update_color(self)
		local color_val = tonumber(self.value)
		if not color_val then --ensure valid number
			local old_vals = {current_color:match("^(%d+) (%d+) (%d+)")}
			self.value = tonumber(self.color_index) and old_vals[tonumber(self.color_index)] or 0
			return false
		end
		
		local red, green, blue = red_text.value, green_text.value, blue_text.value
		local color = red.." "..green.." "..blue
		
		if color == current_color then return false end --color didn't change
		
		last_color = current_color
		set_color(color)
		undo_button.active = "YES"
	end
	
	ok_button = iup.stationbutton{
		title="OK",
		action=function() apply(); HideDialog(dlg) end
	}
	cancel_button = iup.stationbutton{
		title="Cancel",
		action=function() HideDialog(dlg) end
	}
	undo_button = iup.stationbutton{
		title="Undo",
		active="NO",
		action=function(self)
			self.active="NO"
			set_color(last_color)
		end
	}
	default_button = iup.stationbutton{
		title=default_name,
		action=function()
			if default_color == current_color then return false end
			
			last_color = current_color
			set_color(default_color)
			undo_button.active = "YES"
		end
	}
	
	red_text = iup.text{
		size = tostring(Font.Default*3),
		color_index = 1,
		action = function(self, ch, after)
			if ch == 13 then --RETURN
				update_color(self)
				apply()
				HideDialog(dlg)
				return iup.IGNORE
			elseif ch == 27 then --ESCAPE
				HideDialog(dlg)
				return iup.IGNORE
			else
				local number_val = tonumber(self.value)
				if self.value ~= "" and (not number_val or number_val > 255) then return iup.IGNORE end
			end
		end,
		killfocus_cb = function(self)
			update_color(self)
		end,
	}
	
	green_text = iup.text{
		size = tostring(Font.Default*3),
		color_index = 2,
		action = function(self, ch, after)
			if ch == 13 then --RETURN
				update_color(self)
				apply()
				HideDialog(dlg)
				return iup.IGNORE
			elseif ch == 27 then --ESCAPE
				HideDialog(dlg)
				return iup.IGNORE
			else
				local number_val = tonumber(self.value)
				if self.value ~= "" and (not number_val or number_val > 255) then return iup.IGNORE end
			end
		end,
		killfocus_cb = function(self)
			update_color(self)
		end,
	}
	
	blue_text = iup.text{
		size = tostring(Font.Default*3),
		color_index = 3,
		action = function(self, ch, after)
			if ch == 13 then --RETURN
				update_color(self)
				apply()
				HideDialog(dlg)
				return iup.IGNORE
			elseif ch == 27 then --ESCAPE
				HideDialog(dlg)
				return iup.IGNORE
			else
				local number_val = tonumber(self.value)
				if self.value ~= "" and (not number_val or number_val > 255) then return iup.IGNORE end
			end
		end,
		killfocus_cb = function(self)
			update_color(self)
		end,
	}
	
	container = iup.stationhighopacityframe{
		iup.stationhighopacityframebg{
			iup.vbox{
				iup.hbox{iup.fill{}, title_label, iup.fill{}},
				iup.fill{size=2},
				iup.hbox{
					iup.vbox{
						iup.hbox{iup.fill{}, iup.label{title="Red: ", fgcolor="255 0 0"}, red_text},
						iup.hbox{iup.fill{}, iup.label{title="Green: ", fgcolor="0 255 0"}, green_text},
						iup.hbox{iup.fill{}, iup.label{title="Blue: ", fgcolor="0 0 255"}, blue_text},
					},
					iup.fill{size=20},
					iup.vbox{
						iup.fill{},
						iup.stationhighopacitysubframe{previewbox, size="48x48", expand="NO"},
						undo_button,
						alignment="ACENTER",
						gap=2,
					},
				},
				iup.fill{},
				iup.hbox{ok_button, iup.fill{}, default_button, cancel_button, gap=4},
				gap=8,
			},
		},
		expand = "NO",
	}

	setup = function(self)
		undo_button.active = "NO"
		
		local color = self.color or current_color
		default_color = self.default_color or default_color
		last_color = color
		set_color(color)
	end
	
	--tidy up and call color_cb
	apply = function()
		undo_button.active = "NO"
		dlg.color = current_color
		if type(dlg.color_cb)=='function' then dlg:color_cb(current_color) end		
	end
	
	dlg = iup.dialog{
		iup.hbox{
			iup.fill{},
			iup.vbox{
				iup.fill{},
				container,
				iup.fill{}
			},
			iup.fill{},
		},
		fullscreen="YES",
		topmost="YES",
		bgcolor="0 0 0 128 *",
		border="NO",menubox="NO",resize="NO",
		defaultesc=cancelbutton,
		defaultenter=okbutton,
		
		color=current_color,
		default_color=default_color,
		color_cb=type(args.color_cb)=='function' and args.color_cb or nil,
	}

	function dlg:show_cb()
		setup(self)
	end
	
	dlg:map()

	return dlg
end

return cc