local vars = {}
local VARS_ID = "SAS" --section of config to save vars in
vars.NUM_SOUNDS_MAX = 18

vars.is_flash_alert = gkini.ReadInt(VARS_ID, "flash_alert", 1)==1
vars.is_sound_alert = gkini.ReadInt(VARS_ID, "sound_alert", 1)==1
vars.min_snd_replay = 200 --time in ms between playing same sound alert again --TODO save to config
vars.vol_master = tonumber(gkini.ReadString(VARS_ID, "master_volume", '1.0')) --OBSOLETE?

--TODO: Choco function for int2bool
local function int2bool(int)
	if type(int) == 'number' then
		return int==1 and true or false
	elseif type(int) == 'string' then
		return tonumber(int)==1 and true or false
	else return nil end
end

--settings to save to config with value to use when not specified
vars.saved_keys = {
	enable={type='boolean', default=true},
	sound={type='number', default='1'},
	data={type='string', default=''},
	option_enable={type='boolean', default=nil},
	vol={type='number', default=vars.vol_master},
	flash_color={type='string', default='0 0 0'},
}
vars.conv_keys = {boolean=int2bool,number=tonumber,string=tostring} --function to use to convert config data

--Load sounds
vars.sound_name = {}
local sounds = {}
for i=1,vars.NUM_SOUNDS_MAX do
	vars.sound_name[i] = "sound"..i..".ogg"
	--vars.sound_name[-i] = "sound"..i..".wav" --TODO
	
	sounds[i] = gksound.GKLoadSound{soundname=vars.sound_name[i], filename="plugins/SoundAlertSuite/sounds/"..vars.sound_name[i]}
end


--DO NOT CHANGE THE ORDER OF THESE ITEMS! - It corresponds to the same order that settings are saved to the config file
--Use the rank value to change the order shown in GUI (lowest rank first)
--TODO: Multiple lines NYI

	--[[title: text describing the alert, use a table for multiple lines
	   enable: (optional) whether alert is enabled by default, nil treated as true
	   events: table of events that trigger the alert
		 rank: weighted order to display in GUI, lowest first
		 data: (optional) text field to specify additional options, use a table for multiple lines
 option_title: (optional) text field to show next to checkbox to specify additional options, use a table for multiple lines
option_enable: (optional) default state (bool) of checkbox for additional options, use a table for multiple lines
		 func: (optional) function to call when event is triggered to determine if sound should play by returning true (event data passed
		 		to function). If not specified (nil), then sound plays any time one of the events triggers.
				When multiple sounds are used, should return an int indicating which sound to play
		 init: (optional) function to call when alert is enabled/disabled. 1st argument passed is true when alert enabled, false when disabled
		sound: default sound index to play for alert, use a table for multiple sounds
  flash_color: (optional) default color to use for "mute mode" flash alert, using the format: "255 255 255". "0 0 0" or nil disable the flash
  		 help: text to display in the description of this alert in the help dialog

Added later -- Don't define now:
		   id: numeric index for save order (based on order of list below)
		  vol: Playback volume (0 to 1)
		 hide: don't show the alert in the GUI list if true --TODO: NYI]]
vars.alerts = {
	{	title='Chat to non-active channel',
		events={"CHAT_MSG_CHANNEL","CHAT_MSG_CHANNEL_EMOTE"},
		rank=1,
		data="1",
		func='isChatAlert',
		sound=1,
		flash_color="255 255 0",
		help="When someone chats to the channel specified in the data field while you are subscribed to that channel and it is not your active channel.\n\tAdditional parameter: channelid of the channel to monitor",
	},
	{	title='Chat to active channel',
		enable=false,
		events={"CHAT_MSG_CHANNEL_ACTIVE","CHAT_MSG_CHANNEL_EMOTE_ACTIVE"},
		rank=2,
		data="1",
		func='isChatAlert',
		sound=1,
		help="When someone chats to the specified channel while it is active. Activate both this event and the previous to play a sound regardless of whether the channel is active or not.\n\tAdditional parameter: channelid of the channel to monitor",
	},
	{	title='Received private message',
		events={"CHAT_MSG_PRIVATE"},
		rank=10,
		func=nil,
		sound=2,
		help="When you receive a private message (includes PMs from yourself and NPCs).",
	},
	{	title='Buddy entered the game',
		events={"CHAT_MSG_SERVER"},
		rank=20,
		func='isBuddyEntered',
		sound=3,
		help="When one of your buddies logs in.",
	},
	{	title='Buddy left the game',
		events={"CHAT_MSG_SERVER"},
		rank=21,
		func='isBuddyLeft',
		sound=4,
		help="When one of your buddies logs out",
	},
	{	title='Received buddy request',
		events={"CHAT_MSG_SERVER"},
		rank=22,
		func='isBuddyRequest',
		sound=5,
		help="Both when you send/receive a buddy request and when a request is accepted.",
	},
	{	title='Received buddy note',
		events={"CHAT_MSG_INCOMINGBUDDYNOTE"},
		rank=23,
		func=nil,
		sound=6,
		help="When you have an incoming buddy note upon logging in.",
	},
	{	title='Received credits from player',
		events={"CHAT_MSG_SECTORD"},
		rank=30,
		func='isReceivedCredits',
		sound=7,
		flash_color="0 0 255",
		help="When someone sends you credits with the givemoney command (also works when money is sent from NPCs).",
	},
	{	title='Entered NFZ',
		enable=false,
		events={"ENTER_ZONE_NFZ"},
		rank=900,
		func=nil,
		sound=8,
		help="When you enter the No-Fire Zone.",
	},
	{	title='Left NFZ',
		enable=false,
		events={"LEAVE_ZONE_NFZ"},
		rank=901,
		func=nil,
		sound=9,
		help="When you leave the No-Fire Zone.",
	},
	{	title='Mission updated',
		events={"MISSION_NOTIFICATION"},
		rank=300,
		func=nil,
		sound=10,
		help="When your mission is updated ('M' icon appears on your HUD).",
	},
	{	title='Scored a kill',
		enable=false,
		events={"PLAYER_DIED"},
		rank=100,
		option_title="Include NPC kills",
		option_enable=true,
		func='isScoredKill',
		sound=11,
		help="When you kill another player.\n\tEnable the 'Include NPC kills' checkbox to also count NPC kills.",
	},
	{	title='Guild chat',
		enable=false,
		events={"CHAT_MSG_GUILD"},
		rank=11,
		func=nil,
		sound=13,
		help="When you or a guildmate sends a guild chat message.",
	},
	{	title='Group chat',
		enable=false,
		events={"CHAT_MSG_GROUP"},
		rank=12,
		--[[option_title="Include notifications messages",
		option_enable=false,]] --TODO: probably should make it a separate alert since it is a spearate event
		func=nil,
		sound=13,
		help="When you or a group member sends a message to group chat.",
	},
	{	title='New badge',
		events={"PLAYER_RECEIVED_NEW_ACCOMPLISHMENTS"},
		rank=201,
		func=nil,
		sound=14,
		help="When you receive a new badge (or increase the level of an existing one).",
	},
	{	title='Increased license',
		events={"PLAYER_UPDATE_SKILLS"},
		rank=200,
		func='isLicenseIncreased',
		init='saveLicenses',
		sound=16,
		help="When one of your license levels increases.",
	},
	{	title='New key added',
		enable=false,
		events={"KEYADDED"},
		rank=204,
		func='isOutOfAmmo',
		sound=17,
		help="When a new key is added to your keychain.",
	},
	{	title='Out of ammo',
		enable=false,
		events={"INVENTORY_UPDATE"},
		rank=110,
		func='isOutOfAmmo',
		sound=18,
		help="When one of your addons runs out of ammo. The alert occurs on a per-addon basis.",
	},
}

vars.defaults = {} --table of values to use when reseting to defaults
for id,v in ipairs(vars.alerts) do
	local default_values = {}
	for key,info in pairs(vars.saved_keys) do
		v[key] = v[key]==nil and info.default or v[key] --fill in omitted values
		default_values[key] = v[key]
	end
	
	--create table with values to reset to default
	table.insert(vars.defaults, default_values)
end

--read saved settings from config file
for key, info in pairs(vars.saved_keys) do
	local data = unspickle(gkini.ReadString(VARS_ID, key, string.rep("\"\",", #vars.alerts))) --read data into table, default is empty string
	
	--extract entry from data table and convert format
	for id,alert in ipairs(vars.alerts) do
		if key~='data' or alert.data then --ignore config for key data if data not defined in alerts table
			if not data[id] or data[id]=='' then --entry not saved in config
				alert[key] = alert[key] or info.default
			else alert[key] = vars.conv_keys[info.type](data[id]) end
		end
	end
end

--Write all vars data to the config file
function vars.WriteConfig()
	for key, info in pairs(vars.saved_keys) do
		local data = {} --table to write
		
		for id,alert in ipairs(vars.alerts) do
			if info.type=='boolean' then
				data[id] = alert[key] and 1 or 0 --convert boolean values to 1 or 0
			--elseif info.type=='string' and info.default==alert[key] then --TODO need better solution to save space, doesn't work
			--	data[id] = '' --(strings only) write empty string if equal to default value (save space in config file)
			else data[id] = alert[key] or '' end --otherwise write value or empty string if not defined
		end
		
		gkini.WriteString(VARS_ID, key, spickle(data))
	end
	
	gkini.WriteInt(VARS_ID, "flash_alert", vars.is_flash_alert and 1 or 0)
	gkini.WriteInt(VARS_ID, "sound_alert", vars.is_sound_alert and 1 or 0)
end

--Saves key to config as val - currently only works for boolean vars. Returns bool val saved or nil if failed
function vars.WriteVal(key, val)
	key = tostring(key)
	if not key then return end
	if val==nil then return end --exclude false case
	
	local convert = {number=1, string="ON",	boolean=true} --what val has to be to be true given type
	local conv_func = function(v) return v==convert[type(val)] end --converts val to bool
	val = conv_func(val)
	
	vars['is_'..key] = val
	gkini.WriteInt(VARS_ID, key, val and 1 or 0)
	
	return val
end

--All settings to defaults and save them to config -- no confirm dialog
function vars.ResetDefaults()
	for id,alert in ipairs(vars.alerts) do
		for key,value in pairs(vars.defaults[id]) do
		
			--TODO consolidate with ui:Hide() code
			if key=="enable" and alert[key]~=value then --need to register/unregister event
				for _,event_name in ipairs(alert.events) do
					if not alert.enable then --sign flip from ui code
						if not SoundAlertSuite.events_index[event_name] then --event not already registered
							RegisterEvent(SoundAlertSuite, event_name)
							SoundAlertSuite.events_index[event_name] = {}
						end
						table.insert(SoundAlertSuite.events_index[event_name], id)
					else
						--remove all instances of this alert from the event's table
						if SoundAlertSuite.events_index[event_name] then --check table is not nil
							for i=#SoundAlertSuite.events_index[event_name],1,-1 do --have to traverse table backwards for table.remove()
								if SoundAlertSuite.events_index[event_name][i]==id then table.remove(SoundAlertSuite.events_index[event_name], i) end
							end
							
							--unregister event if no more alerts are using the event
							if #SoundAlertSuite.events_index[event_name]==0 then
								SoundAlertSuite.events_index[event_name] = nil
								UnregisterEvent(SoundAlertSuite, event_name)
							end
						end
					end
				end
				
				if alert.init then --call init function because state changed
					SoundAlertSuite[alert.init](alert.enable)
				end
			end
			
			alert[key] = value
		end
	end
	
	vars.is_flash_alert = false
	vars.is_sound_alert = true
	
	vars.WriteConfig()
end

return vars